/********************************************************************
 FileName:	processUSBpacket.c
 Dependencies:	See INCLUDES section
 Processor:	PIC18 and PIC32 USB Microcontrollers
 Hardware:      Simbuck Baseboards and PIC32 USB Starter kit
 Complier:  	Microchip C18 (for PIC18), XC32 (for PIC32)
 Company:	Deben Enterprises Limited


 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
  1.0  First release


/** I N C L U D E S **********************************************************/
#include "simbuck_cfg.h"
#include "HardwareProfile.h"

#if defined(SIMBUCK_BASEBOARD_4550)
 #include <p18cxxx.h>
 #include "system/eeprom/eeprom.h"
#endif

#if defined(SIMBUCK_BASEBOARD_PIC32)
 #include <plib.h>
#endif

#include "USB/usb.h"
#include "USB/usb_function_generic.h"
#include "processUSBpacket.h"
#include "watch.h"

#if defined(SIMBLOCK_DUAL_CAN)
  #include "HardwareProfile - SimblockDualCAN.h"
  #include "../system/CAN/CAN2515drv.h"
  #include "../system/CAN/CAN2515user.h"
  #include "CAN/processUSBpacketCAN.h"
#endif

/** V A R I A B L E S ********************************************************/
#pragma udata
extern unsigned char OUTPacket[64];		//User application buffer for receiving and holding OUT packets sent from the host
extern unsigned char INPacket[64];		//User application buffer for sending IN packets to the host
extern USB_HANDLE USBGenericInHandle;
extern USB_HANDLE USBGenericOutHandle;
extern MCU_REGS MCU_regs;

/** D E C L A R A T I O N S **************************************************/
#pragma code


/******************************************************************************
 * Function:        void ProcessUSB(void)
 *
 * User USB message structure
 *
 * PreCondition:    USB communications has been established
 *
 * Input:           A Correctly formatted byte stream
 *                           <REQ><CTRL><RECLEN><DATA><DATA..>
 *                  OUTPacket [0]  [1]    [2]    [3]   [..]
 *
 * Output:          The response bytes to the request
 *                          <REQ><CTRL><RECLEN><ERR><DATA><DATA..>
 *                  INPacket [0]  [1]    [2]    [3]  [4]   [5]
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for user USB message
 * 		    Structure(s).
 *
 *
 * Note:            The user is expected to populate this function with user
 *		    relevant data over the USB connection
 */


void ProcessUSB(void)
{
    char i;
    char offset;
    char reclen;
    char bytescnt = 0;

   // echo back the request
    INPacket[0] = OUTPacket[0];
    INPacket[1] = OUTPacket[1];
    INPacket[2] = 0xFF;
    INPacket[3] = 0xFF;

    switch(OUTPacket[0])	//Data arrived
        {
        case DEVICE_VERSION: // Version request 0x10
        {
         switch (OUTPacket[1])
            {
             case 1: // Old method

             break;

#if defined(SIMBUCK_BASEBOARD_4550)
  #if defined(HID_BOOTLOADER)
                case 2: // Firmware

                    INPacket[4] = STACK_YEAR_USB;
                    INPacket[5] = STACK_MONTH_USB;
                    INPacket[6] = STACK_DAY_USB;

                    INPacket[7] = BOOTLOAD_TYPE;

                    INPacket[8] = SIMBUCK_BOOT_MAJOR;
                    INPacket[9] = SIMBUCK_BOOT_MINOR;
                    INPacket[10] = SIMBUCK_BOOT_DOT;

                    bytescnt += 11;
  #endif
                    INPacket[11] = DEBUG_VERSION_BASE;
                    INPacket[12] = MINOR_VERSION_BASE;
                    INPacket[13] = MAJOR_VERSION_BASE;

                    bytescnt +=3;

   #if defined(MY_4550_DEVICE)
                    INPacket[14] = DEBUG_VERSION_DEVICE;
                    INPacket[15] = MINOR_VERSION_DEVICE;
                    INPacket[16] = MAJOR_VERSION_DEVICE;

                    bytescnt +=3;
   #endif
#endif
                    INPacket[2] = bytescnt;
                    INPacket[3] = SUCCESS;

                    break;

                   }
                 }
            break;

        case READ_EEPROM: // 0x14 Read the EEPROM data from PC application.
        { // we use the second byte for the data segment
            offset = OUTPacket[1];
            INPacket[4] = ReadFromEEPROM(offset);

            INPacket[2] = 0x05;
            INPacket[3] = SUCCESS;
        }
            break;

        case WRITE_EEPROM: // 0x15 Write the EEPROM data from PC application.
        { // we use the second byte for the data segment
            offset = OUTPacket[1];
            reclen = OUTPacket[2];

            for (i = 0; i < 6; i++) {
                WriteToEEPROM(OUTPacket[2 + i], offset + i);
                INPacket[4 + i] = OUTPacket[2 + i];
            }

            INPacket[2] = 0x0A;
            INPacket[3] = SUCCESS;

        }
            break;

        case GET_BLOCK: // 0x16 Read Config record from EEPROM
        { // we use the second byte for the data segment
            // we use 3rd byte to define number bytes to read
            offset = OUTPacket[1];
            reclen = OUTPacket[2];

            for (i = 0; i < reclen; i++) {
                INPacket[4 + i] = ReadFromEEPROM(offset + i);
            }

            INPacket[2] = reclen + 3;
            INPacket[3] = SUCCESS;
        }
            break;


         /* self test (Simbuck-JPF) */
          case GET_FUSES: //0x20
            {
             TBLPTR = (unsigned short long) 0x300000;

             for (i = 0; i < 14; i++) {
             _asm
		tblrdpostinc
 	    _endasm

             INPacket[i+4] = TABLAT;

             }

             TBLPTR = (unsigned short long) 0x3FFFFE;
              _asm
		tblrdpostinc
             _endasm
              i++;
             INPacket[i+4] = TABLAT;

              _asm
		tblrdpostinc
             _endasm
              i++;
             INPacket[i+4] = TABLAT;

             INPacket[2] = i+4;
             INPacket[3] = SUCCESS;
            }

         break;


        case GET_USER_STATUS: //0x22 the register setup
         {
            INPacket[4] = MCU_regs._TRISE;
            INPacket[5] = MCU_regs._TRISD;
            INPacket[6] = MCU_regs._TRISC;
            INPacket[7] = MCU_regs._TRISB;
            INPacket[8] = MCU_regs._TRISA;
            INPacket[9] = MCU_regs._PIE1;
            INPacket[10] = MCU_regs._PIE2;
            INPacket[11] = MCU_regs._PIR1;
            INPacket[12] = MCU_regs._PIR2;
            INPacket[13] = MCU_regs._IPR1;
            INPacket[14] = MCU_regs._IPR2;
            INPacket[15] = MCU_regs._RCON;

            INPacket[16] = MCU_regs._INTCON;
            INPacket[17] = MCU_regs._INTCON2;
            INPacket[18] = MCU_regs._INTCON3;

            INPacket[2] = 19;
            INPacket[3] = SUCCESS;
        }
            break;

        case GET_PORTS: //0x23
        {
            INPacket[4] = MCU_regs._PORTE;
            INPacket[5] = MCU_regs._PORTD;
            INPacket[6] = MCU_regs._PORTC;
            INPacket[7] = MCU_regs._PORTB;
            INPacket[8] = MCU_regs._PORTA;
            INPacket[9] = MCU_regs._LATE;
            INPacket[10] = MCU_regs._LATD;
            INPacket[11] = MCU_regs._LATC;
            INPacket[12] = MCU_regs._LATB;
            INPacket[13] = MCU_regs._LATA;

            INPacket[2] = 14;
            INPacket[3] = SUCCESS;
        }
            break;

            case READ_PORTS:  // 0x3A
            #if defined(SIMBUCK_BASEBOARD_4550)
                INPacket[4] = PORTA;
            #endif
                INPacket[5] = PORTB;
                INPacket[6] = PORTC;
                INPacket[7] = PORTD;
                INPacket[8] = PORTE;

                INPacket[2] = 0x09;
                INPacket[3] = SUCCESS;

            break;

           #if defined(SIMBUCK_BASEBOARD_4550)
             case MONITOR:  // 0x3B
              INPacket[4] = MCU_regs._PORTE;
              INPacket[5] = MCU_regs._PORTD;
              INPacket[6] = MCU_regs._PORTC;
              INPacket[7] = MCU_regs._PORTB;
              INPacket[8] = MCU_regs._PORTA;
              INPacket[9] = MCU_regs._LATE;
              INPacket[10] = MCU_regs._LATD;
              INPacket[11] = MCU_regs._LATC;
              INPacket[12] = MCU_regs._LATB;
              INPacket[13] = MCU_regs._LATA;

              INPacket[14] = MCU_regs._TRISE;
              INPacket[15] = MCU_regs._TRISD;
              INPacket[16] = MCU_regs._TRISC;
              INPacket[17] = MCU_regs._TRISB;
              INPacket[18] = MCU_regs._TRISA;
              INPacket[19] = MCU_regs._PIE1;
              INPacket[20] = MCU_regs._PIE2;
              INPacket[21] = MCU_regs._PIR1;
              INPacket[22] = MCU_regs._PIR2;
              INPacket[23] = MCU_regs._IPR1;
              INPacket[24] = MCU_regs._IPR2;
              INPacket[25] = MCU_regs._RCON;

            INPacket[26] = MCU_regs._INTCON;
            INPacket[27] = MCU_regs._INTCON2;
            INPacket[28] = MCU_regs._INTCON3;

                INPacket[2] = 0x29;
                INPacket[3] = SUCCESS;

             break;

            #if defined(SIMBUCK_BASEBOARD_4550)
            case RESET_DEVICE:                  // Copied direct from HID Bootloader
	       {
		 UCONbits.SUSPND = 0;		//Disable USB module
		 UCON = 0x00;			//Disable USB module
		//And wait awhile for the USB cable capacitance to discharge down to disconnected (SE0) state.
		//Otherwise host might not realize we disconnected/reconnected when we do the reset.
		//A basic for() loop decrementing a 16 bit number would be simpler, but seems to take more code space for
		//a given delay.  So do this instead:
		for(i = 0; i < 0xFF; i++)
		  {
		    WREG = 0xFF;
		    while(WREG)
			{
			 WREG--;
			 _asm
			 bra	0	//Equivalent to bra $+2, which takes half as much code as 2 nop instructions
			 bra	0	//Equivalent to bra $+2, which takes half as much code as 2 nop instructions
			 _endasm
			}
		   }
		 Reset();
		}
		break;
            #endif

            default:

            #if defined(SIMBLOCK_DUAL_CAN)
             // process dualCAN specific messages
            ProcessUSBdualCAN();

            #endif

             if(INPacket[3] != SUCCESS)
             {
              INPacket[3] = ERROR_RESPONSE;
              INPacket[4] = NOT_SUPPORTED;
              INPacket[2] = 0x05;
             }
             
        }


	//Now check to make sure no previous attempts to send data to the host are still pending.  If any attemps are still
	//pending, we do not want to write to the endpoint 1 IN buffer again, until the previous transaction is complete.
	//Otherwise the unsent data waiting in the buffer will get overwritten and will result in unexpected behavior.
        if(!USBHandleBusy(USBGenericInHandle))
	  {
	           //The endpoint was not "busy", therefore it is safe to write to the buffer and arm the endpoint.
	           //The USBGenWrite() function call "arms" the endpoint (and makes the handle indicate the endpoint is busy).
	           //Once armed, the data will be automatically sent to the host (in hardware by the SIE) the next time the
	           //host polls the endpoint.  Once the data is successfully sent, the handle (in this case USBGenericInHandle)
	           //will indicate the the endpoint is no longer busy.
		   USBGenericInHandle = USBGenWrite(USBGEN_EP_NUM,(BYTE*)&INPacket,USBGEN_EP_SIZE);
        }


 }//end ProcessIO







/** EOF user.c ***************************************************************/
